#!/usr/bin/perl
#
# QuadToneRIP Printer Setup and Curve Installation Script
#
#   Copyright (c) 2008 -- Roy V. Harrington  
#   Email:     quadtone@harrington.com
#   Website:   www.quadtonerip.com
#

my $pr_model = $0;
   $pr_model =~ s/^.*\///;
   $pr_model =~ s/^Install//i;
   $pr_model =~ s/\.command$//ig;
   $pr_model =~ s/\.pl$//g;

my $pr_base1 = $pr_model;
   $pr_base1 =~ s/[-_=].*$//;

my $pr_base2 = $pr_base1;

if ($pr_model =~ m/^(.*)=(.*)$/) {
    $pr_model = $1;
    $pr_base2 = $2;
}

my @profiles = @ARGV;

my $printer  = "Quad" . $pr_model;
my $printer2 = $printer;
   $printer2 =~ s/-/_/g;    # check for OS making it an underscore

my $ppd_model= "Quad" . $pr_base2 . ".ppd.gz";

my $dir_name = $0;
   $dir_name =~ s/\/[^\/]+$//;
   
chdir $dir_name;

my $qtrbase = "/Library/Printers/QTR";

my $CUPSDATADIR		=	`cups-config --datadir`;
my $CUPSSERVROOT	=	`cups-config --serverroot`;
my $CUPSSERVBIN		=	`cups-config --serverbin`;

chomp $CUPSDATADIR;
chomp $CUPSSERVROOT;
chomp $CUPSSERVBIN;

my $QUADTONE		=	$qtrbase  . "/quadtone";
my $PPDDIR		=	$CUPSSERVROOT . "/ppd";
my $USB			=	$CUPSSERVBIN  . "/backend/usb";

my $MODEL		=	${CUPSDATADIR}. "/model";
my $URI;
my $LOC;

if ( ! -x "$qtrbase/filter/quadtoneprint") {
	print "\n********************************************************************\n\n";
	print   "ALERT! **** QuadToneRIP does not appear to have been installed! ****\n";
	print "\n********************************************************************\n\n";
	exit 1;
}

my $osv = `uname -r`;

if (-f "$PPDDIR/$printer2.ppd" ||  $osv =~ m/^6\./ ) {
	$printer = $printer2;
}

my $PPDFILE = "$PPDDIR/$printer.ppd";

if (-f "$PPDFILE" && `grep -q QuadToneRIP "$PPDFILE"`) {
	$ppd_model =`grep PCFileName "$PPDFILE" | sed -e 's/^.*Quad/Quad/' -e 's/ppd.*\$/ppd.gz/'`;
	chomp $ppd_model;
}

if ( ! -f "$MODEL/C/$ppd_model") {
	print "\n*********************************************************************\n\n";
	print   "ALERT! **** QuadToneRIP does not support printer $pr_base2 ****\n";
	print "\n*********************************************************************\n\n";
	exit 1;
}

if ( ! -f "$PPDFILE" || `lpstat -v $printer` =~ m/dev.null/) {
    open (USBLIST, "$USB|");
    while ($URI = <USBLIST>) {
	chomp $URI;
	$URI =~ s/^direct *//;
	$URI =~ s/ .*$//;
	last if ($URI =~ m/usb:.*EPSON.*$pr_base1/);
    }
    close USBLIST;
    $LOC = "USB Printer";
    if ($URI eq "") {
		print "\n*********************************************************************\n\n";
		print   "Cannot find an Epson USB printer with the name $pr_base1\n\n";
		print   "The printer must be turned on, plugged in to computer and ready\n";
		print   "Resetting the printer may be necessary.\n\n";
		print   "The firewire port cannot be used for QuadToneRIP.  If you are already using\n";
		print   "firewire you must switch over to USB.  Do not plug in both USB and firewire.\n\n";
		print   "For network printers, please enter an IPP address for your printer.\n";
		for (;;) {
		    print   "The address is a 4 part number such as 192.38.103.34\n",
				"Enter IPP number or <return> to cancel:  ";
		
		    my $ipp = <STDIN>;
		    chomp $ipp;
		    if ( $ipp =~ m/^$/ ) {
			print "\nInstall cancelled.\n\n";
			exit 1;
		    }
		    if ( $ipp =~ m/^\d+\.\d+\.\d+\.\d+$/ ) {
			$URI = "lpd://$ipp";
			$LOC = "Network Printer IPP = $ipp"; 
			last;
		    } else {
			print "\nInvalid IPP number\n\n";
		    }
		}
    }

    print "Creating printer named: $printer\n";
    system "lpadmin -p $printer -m C/$ppd_model -v \"$URI\" -L \"$LOC\" -E";
} else {
    system "lpadmin -p $printer -m C/$ppd_model -E";
}

if ( ! -d "$QUADTONE/$printer" ) {
    mkdir "$QUADTONE/$printer", 0777;
}

if ( ! -d "$QUADTONE/$printer") {
	print "\n*********************************************************************\n\n";
	print   "ALERT! **** Unable to create curve folder $QUADTONE/$printer ****\n";
	print "\n*********************************************************************\n\n";
	exit 1;
}

for my $CURVE ( @ARGV || <*> ) {
  my $QUAD = $CURVE;
  $QUAD =~ s/(.txt$|.qidf$)/.quad/;
  if ( ! -e "$QUADTONE/$printer/$QUAD" || -M "$QUADTONE/$printer/$QUAD" > -M "$CURVE" ) {
	if ( "$CURVE" =~ m/(.txt$|.qidf$)/ ) {
	  system "PRINTER=$printer INSTALL_CURVE=NO $qtrbase/bin/quadprofile \"$CURVE\""
	}
	if ( "$CURVE" =~ m/.quad$/ ) {
	  system "cp \"$CURVE\" $QUADTONE/$printer"
	}
  }
}

system "$qtrbase/bin/quadcurves $printer";

